@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Collector.pm,v 1.8 2005/08/11 12:32:26 ssinyagin Exp $
+# $Id: Collector.pm,v 1.11 2006/08/06 18:30:33 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -36,6 +36,18 @@ BEGIN
     }
 }
 
+# Executed once after the fork. Here modules can launch processing threads
+sub initThreads
+{
+    foreach my $key ( %Torrus::Collector::initThreadsHandlers )
+    {
+        if( ref( $Torrus::Collector::initThreadsHandlers{$key} ) )
+        {
+            &{$Torrus::Collector::initThreadsHandlers{$key}}();
+        }
+    }
+}
+
 
 ## One collector module instance holds all leaf tokens which
 ## must be collected at the same time.
@@ -62,6 +74,12 @@ sub new
     foreach my $storage_type ( keys %Torrus::Collector::storageTypes )
     {
         $self->{'storage'}{$storage_type} = {};
+        
+        my $storage_string = $storage_type . '-storage';
+        if( ref( $Torrus::Collector::initStorage{$storage_string} ) )
+        {
+            &{$Torrus::Collector::initStorage{$storage_string}}($self);
+        }
     }
 
     $self->{'tree_name'} = $options{'-TreeName'};
@@ -658,6 +676,10 @@ sub cacheCollectors
                                  ( $ctoken, 'collector-timeoffset' ) );
 
             $data->{'db_tokens'}->put( $ctoken, $period.':'.$offset );
+            if( not exists( $data->{'targets'}{$period}{$offset} ) )
+            {
+                $data->{'targets'}{$period}{$offset} = [];
+            }
             push( @{$data->{'targets'}{$period}{$offset}}, $ctoken );
         }
     }
