@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: SNMP_Params.pm,v 1.3 2005/11/22 16:31:18 ssinyagin Exp $
+# $Id: SNMP_Params.pm,v 1.4 2006/11/26 22:39:03 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::SNMP_Params;
@@ -26,6 +26,8 @@ my %validatorLeafParams =
     (
      'snmp-host'          => undef,
      'snmp-port'          => undef,
+     '+snmp-localaddr'    => undef,
+     '+snmp-localport'    => undef,
      '+domain-name'       => undef,     
      'snmp-object'        => undef,
      'snmp-version'       => { '1'  => { 'snmp-community'     => undef },
@@ -64,6 +66,8 @@ my %admInfoLeafParams =
     (
      'snmp-host'          => undef,
      'snmp-port'          => undef,
+     'snmp-localaddr'     => undef,
+     'snmp-localport'     => undef,
      'domain-name'        => undef,
      'snmp-community'     => undef,
      'snmp-username'      => undef,
@@ -87,6 +91,8 @@ my %admInfoParamCategories =
     (
      'snmp-host'          => 'SNMP',
      'snmp-port'          => 'SNMP',
+     'snmp-localaddr'     => 'SNMP',
+     'snmp-localport'     => 'SNMP',
      'domain-name'        => 'SNMP',
      'snmp-community'     => 'SNMP',
      'snmp-username'      => 'SNMP',
