@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: ConfigTree.pm,v 1.3 2005/05/04 13:19:52 ssinyagin Exp $
+# $Id: ConfigTree.pm,v 1.5 2006/12/03 12:46:08 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -67,6 +67,42 @@ sub new
 
     if( $options{'-WriteAccess'} )
     {
+        $self->{'is_writing'} = 1;
+        
+        # Acquire exlusive lock on the database and set the compiling flag
+        {
+            my $ok = 1;
+            my $key = 'compiling:' . $self->{'treename'};
+            my $cursor = $self->{'db_config_instances'}->cursor( -Write => 1 );
+            my $compilingFlag =
+                $self->{'db_config_instances'}->c_get( $cursor, $key );
+            if( $compilingFlag )
+            {
+                if( $options{'-ForceWriter'} )
+                {
+                    Warn('Another compiler process is probably still ' .
+                         'running. This may lead to an unusable ' .
+                         'database state');
+                }
+                else
+                {
+                    Error('Another compiler is running for the tree ' .
+                          $self->{'treename'});
+                    $ok = 0;
+                }
+            }
+            else
+            {
+                $self->{'db_config_instances'}->c_put( $cursor, $key, 1 );
+            }
+            undef $cursor;
+            if( not $ok )
+            {
+                return undef;
+            }
+            $self->{'iam_writer'} = 1;
+        }
+
         if( not $options{'-NoDSRebuild'} )
         {
             $dsConfInstance = sprintf( '%d', ( $dsConfInstance + 1 ) % 2 );
@@ -82,11 +118,6 @@ sub new
                                          -WriteAccess => 1 );
     defined( $self->{'db_readers'} ) or return( undef );
 
-    if( $options{'-WriteAccess'} )
-    {
-        $self->{'is_writing'} = 1;
-    }
-
     $self->{'db_dsconfig'} =
         new Torrus::DB('ds_config_' . $dsConfInstance,
                      -Subdir => $self->{'treename'},  -Btree => 1,
@@ -193,7 +224,18 @@ sub DESTROY
 
     Debug('Destroying ConfigTree object');
 
-    $self->clearReader();
+    if( $self->{'iam_writer'} )
+    {
+        # Acquire exlusive lock on the database and clear the compiling flag
+        my $cursor = $self->{'db_config_instances'}->cursor( -Write => 1 );
+        $self->{'db_config_instances'}->c_put
+            ( $cursor, 'compiling:' . $self->{'treename'}, 0 );
+        undef $cursor;
+    }
+    else
+    {
+        $self->clearReader();
+    }
 
     undef $self->{'db_dsconfig'};
     undef $self->{'db_otherconfig'};
@@ -259,7 +301,7 @@ sub waitReaders
         my $cursor = $self->{'db_readers'}->cursor();
         while( my ($key, $val) = $self->{'db_readers'}->next( $cursor ) )
         {
-            my( $timestamp, $dsInst, $otherInst ) = split( ':', $val );
+            my( $timestamp, $dsInst, $otherInst ) = split( /:/o, $val );
             if( $dsInst == $self->{'ds_config_instance'} or
                 $otherInst == $self->{'other_config_instance'} )
             {
@@ -301,7 +343,8 @@ sub waitReaders
                 while( my ($key, $val) =
                        $self->{'db_readers'}->next( $cursor ) )
                 {
-                    my( $timestamp, $dsInst, $otherInst ) = split( ':', $val );
+                    my( $timestamp, $dsInst, $otherInst ) =
+                        split( /:/o, $val );
                     if( $dsInst == $self->{'ds_config_instance'} or
                         $otherInst == $self->{'other_config_instance'} )
                     {
@@ -346,7 +389,7 @@ sub splitPath
     while( length($path) > 0 )
     {
         my $node;
-        $path =~ s/^([^\/]*\/?)//; $node = $1;
+        $path =~ s/^([^\/]*\/?)//o; $node = $1;
         push(@ret, $node);
     }
     return @ret;
@@ -356,7 +399,7 @@ sub nodeName
 {
     my $self = shift;
     my $path = shift;
-    $path =~ s/.*\/([^\/]+)\/?$/$1/;
+    $path =~ s/.*\/([^\/]+)\/?$/$1/o;
     return $path;
 }
 
@@ -437,7 +480,7 @@ sub getAliases
     my $self = shift;
     my $token = shift;
 
-    return split( ',', $self->{'db_dsconfig'}->get( 'ar:'.$token ) );
+    return split( /,/o, $self->{'db_dsconfig'}->get( 'ar:'.$token ) );
 }
 
 
@@ -635,7 +678,7 @@ sub getParamNames
 
     my $db = $fromDS ? $self->{'db_dsconfig'} : $self->{'db_otherconfig'};
 
-    return split( ',', $db->get( 'Pl:'.$name ) );
+    return split( /,/o, $db->get( 'Pl:'.$name ) );
 }
 
 sub getParams
@@ -684,7 +727,7 @@ sub getChildren
 
         if( defined $children )
         {
-            my @clist = split(',', $children);
+            my @clist = split(/,/o, $children);
             map { $self->{'parentcache'}{$_} = $token; } @clist;
             return @clist;
         }
@@ -704,7 +747,7 @@ sub getNodesByPattern
     my $self = shift;
     my $pattern = shift;
 
-    if( $pattern !~ /^\// )
+    if( $pattern !~ /^\//o )
     {
         Error("Incorrect pattern: $pattern");
         return undef;
@@ -717,9 +760,9 @@ sub getNodesByPattern
 
         # Cut the trailing slash, if any
         my $patternname = $nodepattern;
-        $patternname =~ s/\/$//;
+        $patternname =~ s/\/$//o;
 
-        if( $patternname =~ /\W/ )
+        if( $patternname =~ /\W/o )
         {
             foreach my $candidate ( @retlist )
             {
@@ -728,8 +771,8 @@ sub getNodesByPattern
                 {
                     # Cut the trailing slash and leading path
                     my $childname = $self->path($child);
-                    $childname =~ s/\/$//;
-                    $childname =~ s/.*\/([^\/]+)$/$1/;
+                    $childname =~ s/\/$//o;
+                    $childname =~ s/.*\/([^\/]+)$/$1/o;
                     if( $childname =~ $patternname )
                     {
                         push( @next_retlist, $child );
@@ -767,7 +810,7 @@ sub getRelative
     my $token = shift;
     my $relPath = shift;
 
-    if( $relPath =~ /^\// )
+    if( $relPath =~ /^\//o )
     {
         return $self->token( $relPath );
     }
@@ -780,9 +823,9 @@ sub getRelative
 
         while( length( $relPath ) > 0 )
         {
-            if( $relPath =~ /^\.\.\// )
+            if( $relPath =~ /^\.\.\//o )
             {
-                $relPath =~ s/^\.\.\///;
+                $relPath =~ s/^\.\.\///o;
                 if( $token ne $self->token('/') )
                 {
                     $token = $self->getParent( $token );
@@ -791,7 +834,7 @@ sub getRelative
             else
             {
                 my $childName;
-                $relPath =~ s/^([^\/]*\/?)//; $childName = $1;
+                $relPath =~ s/^([^\/]*\/?)//o; $childName = $1;
                 my $path = $self->path( $token );
                 $token = $self->token( $path . $childName );
                 if( not defined $token )
@@ -867,7 +910,7 @@ sub getViewNames
     my $self = shift;
     my $vlist = $self->{'db_otherconfig'}->get( 'V:' );
 
-    return defined($vlist) ? split(',', $vlist) : ();
+    return defined($vlist) ? split(/,/o, $vlist) : ();
 }
 
 
@@ -884,7 +927,7 @@ sub getMonitorNames
     my $self = shift;
     my $mlist = $self->{'db_otherconfig'}->get( 'M:' );
 
-    return defined($mlist) ? split(',', $mlist) : ();
+    return defined($mlist) ? split(/,/o, $mlist) : ();
 }
 
 sub monitorExists
@@ -900,7 +943,7 @@ sub getActionNames
     my $self = shift;
     my $alist = $self->{'db_otherconfig'}->get( 'A:' );
 
-    return defined($alist) ? split(',', $alist) : ();
+    return defined($alist) ? split(/,/o, $alist) : ();
 }
 
 
@@ -950,7 +993,7 @@ sub getTsets
 {
     my $self = shift;
     my $list = $self->{'db_sets'}->get('S:');
-    return defined($list) ? split(',', $list) : ();
+    return defined($list) ? split(/,/o, $list) : ();
 }
 
 sub tsetMembers
@@ -959,7 +1002,7 @@ sub tsetMembers
     my $tset = shift;
 
     my $list = $self->{'db_sets'}->get('s:'.$tset);
-    return defined($list) ? split(',', $list) : ();
+    return defined($list) ? split(/,/o, $list) : ();
 }
 
 sub tsetAddMember
@@ -995,7 +1038,7 @@ sub getDefinitionNames
     my $self = shift;
     my $dlist = $self->{'db_dsconfig'}->get( 'D:' );
 
-    return defined($dlist) ? split(',', $dlist) : ();
+    return defined($dlist) ? split(/,/o, $dlist) : ();
 }
 
 
