@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Validator.pm,v 1.7 2005/10/19 14:50:10 ssinyagin Exp $
+# $Id: Validator.pm,v 1.8 2006/11/03 14:04:39 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -44,8 +44,10 @@ my %rrd_params =
 
 my %rrdmulti_params = ( 'ds-names' => undef );
 
-my %collector_params =
+# Plugins might need to add a new storage type
+our %collector_params =
     (
+     'collector-type'  => undef,
      '@storage-type'   => {
          'rrd' => {
              'data-file'              => undef,
@@ -86,7 +88,8 @@ my %collector_params =
      );
 
 
-my %leaf_params =
+# Plugins might in theory create new datasource types
+our %leaf_params =
     ('ds-type' => {'rrd-file' => \%rrd_params,
                    'rrd-multigraph' => \%rrdmulti_params,
                    'collector' => \%collector_params},
