@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Writer.pm,v 1.6 2006/02/23 12:07:43 ssinyagin Exp $
+# $Id: Writer.pm,v 1.11 2006/12/03 12:46:09 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 #
@@ -28,6 +28,7 @@ our @ISA=qw(Torrus::ConfigTree);
 
 use Torrus::Log;
 use Torrus::TimeStamp;
+use Torrus::SiteConfig;
 use Torrus::ServiceID;
     
 use strict;
@@ -44,6 +45,11 @@ foreach my $param ( @remspace )
     $Torrus::ConfigTree::Writer::remove_space{$param} = 1;
 }
 
+our %multigraph_remove_space =
+    ('ds-expr-' => 1,
+     'graph-legend-' => 0);
+
+
 # instance of Torrus::ServiceID object, if needed
 my $srvIdParams;
 
@@ -55,9 +61,16 @@ sub new
     my $class = ref($proto) || $proto;
     $options{'-WriteAccess'} = 1;
     my $self  = $class->SUPER::new( %options );
+    if( not defined( $self ) )
+    {
+        return undef;
+    }
+    
     bless $self, $class;
 
     $self->{'viewparent'} = {};
+    $self->{'mayRunCollector'} =
+        Torrus::SiteConfig::mayRunCollector( $self->treeName() );
 
     return $self;
 }
@@ -82,7 +95,7 @@ sub setParam
 
     if( $Torrus::ConfigTree::Writer::remove_space{$param} )
     {
-        $value =~ s/\s+//g;
+        $value =~ s/\s+//go;
     }
 
     $self->{'paramcache'}{$name}{$param} = $value;
@@ -99,7 +112,7 @@ sub setNodeParam
 
     if( $Torrus::ConfigTree::Writer::remove_space{$param} )
     {
-        $value =~ s/\s+//g;
+        $value =~ s/\s+//go;
     }
 
     $self->{'paramcache'}{$name}{$param} = $value;
@@ -155,7 +168,7 @@ sub addChild
         {
             $nodeType = 2; # alias
         }
-        elsif( $childname =~ /\/$/ )
+        elsif( $childname =~ /\/$/o )
         {
             $nodeType = 0; # subtree
         }
@@ -179,9 +192,9 @@ sub setAlias
     my $iamLeaf = $self->isLeaf($token);
 
     # TODO: Add more verification here
-    if( not defined($apath) or $apath !~ /^\// or
-        ( not $iamLeaf and $apath !~ /\/$/ ) or
-        ( $iamLeaf and $apath =~ /\/$/ ) )
+    if( not defined($apath) or $apath !~ /^\//o or
+        ( not $iamLeaf and $apath !~ /\/$/o ) or
+        ( $iamLeaf and $apath =~ /\/$/o ) )
     {
         my $path = $self->path($token);
         Error("Incorrect alias at $path: $apath"); $ok = 0;
@@ -299,7 +312,7 @@ sub isTrueVar
         if( defined( $value ) )
         {
             if( $value eq 'true' or
-                $value =~ /^\d+$/ and $value )
+                $value =~ /^\d+$/o and $value )
             {
                 $ret = 1;
             }
@@ -374,7 +387,7 @@ sub postProcessNodes
         my $tsets = $self->getNodeParam( $token, 'tokenset-member' );
         if( defined( $tsets ) )
         {
-            foreach my $tset ( split(',', $tsets) )
+            foreach my $tset ( split(/,/o, $tsets) )
             {
                 my $tsetName = 'S'.$tset;
                 if( not $self->tsetExists( $tsetName ) )
@@ -398,27 +411,34 @@ sub postProcessNodes
                 # Expand parameter substitutions in multigraph leaves
                 
                 my @dsNames =
-                    split(',', $self->getNodeParam($token, 'ds-names') );
+                    split(/,/o, $self->getNodeParam($token, 'ds-names') );
                 
                 foreach my $dname ( @dsNames )
                 {
-                    foreach my $param ( 'ds-expr-' )
+                    foreach my $param ( 'ds-expr-', 'graph-legend-' )
                     {
                         my $dsParam = $param . $dname;
                         my $value = $self->getNodeParam( $token, $dsParam );
-                        my $newValue = $value;
-                        $newValue =~ s/\s+//g;
-                        $newValue =
-                            $self->expandSubstitutions( $token, $dsParam,
-                                                        $newValue );
-                        if( $newValue ne $value )
+                        if( defined( $value ) )
                         {
-                            $self->setNodeParam( $token, $dsParam, $newValue );
+                            my $newValue = $value;
+                            if( $multigraph_remove_space{$param} )
+                            {
+                                $newValue =~ s/\s+//go;
+                            }
+                            $newValue =
+                                $self->expandSubstitutions( $token, $dsParam,
+                                                            $newValue );
+                            if( $newValue ne $value )
+                            {
+                                $self->setNodeParam( $token, $dsParam,
+                                                     $newValue );
+                            }
                         }
                     }
                 }
             }
-            elsif( $dsType eq 'collector' )
+            elsif( $dsType eq 'collector' and $self->{'mayRunCollector'} )
             {
                 my $dispersed =
                     $self->getNodeParam($token,
@@ -479,7 +499,7 @@ sub postProcessNodes
 
                 my $storagetypes =
                     $self->getNodeParam( $token, 'storage-type' );
-                foreach my $stype ( split(',', $storagetypes) )
+                foreach my $stype ( split(/,/o, $storagetypes) )
                 {
                     if( $stype eq 'ext' )
                     {
