@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: XMLCompiler.pm,v 1.2 2005/01/25 13:46:33 ssinyagin Exp $
+# $Id: XMLCompiler.pm,v 1.4 2006/12/03 12:46:09 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -41,6 +41,11 @@ sub new
     $options{'-Rebuild'} = 1;
 
     my $self  = $class->SUPER::new( %options );
+    if( not defined( $self ) )
+    {
+        return undef;
+    }
+
     bless $self, $class;
 
     if( $options{'-NoDSRebuild'} )
@@ -198,14 +203,14 @@ sub compile_params
         else
         {
             # Remove spaces in the head and tail.
-            $value =~ s/^\s+//;
-            $value =~ s/\s+$//;
+            $value =~ s/^\s+//o;
+            $value =~ s/\s+$//o;
 
             if( $param eq 'legend' )
             {
                 # Remove space around delimiters
-                $value =~ s/\s*\:\s*/\:/g;
-                $value =~ s/\s*\;\s*/\;/g;
+                $value =~ s/\s*\:\s*/\:/go;
+                $value =~ s/\s*\;\s*/\;/go;
             }
             elsif( $param eq 'ds-type' and $value eq 'RRDfile' )
             {
@@ -267,7 +272,7 @@ sub expand_name
     my $childname = shift;
     if( index($childname, '$PARENT') > -1 )
     {
-        my @pathparts = split('/', $parentpath);
+        my @pathparts = split(/\//o, $parentpath);
         my $parentname = pop @pathparts;
         $childname =~ s/\$PARENT/$parentname/g;
     }
@@ -279,8 +284,8 @@ sub validate_nodename
     my $self = shift;
     my $name = shift;
 
-    return ( $name =~ /^[0-9A-Za-z_\-\.]+$/ and
-             $name !~ /\.\./ );
+    return ( $name =~ /^[0-9A-Za-z_\-\.]+$/o and
+             $name !~ /\.\./o );
 }
 
 sub compile_subtrees
@@ -323,7 +328,7 @@ sub compile_subtrees
     foreach my $alias ( $node->getChildrenByTagName('alias') )
     {
         my $apath = $alias->textContent();
-        $apath =~ s/\s+//mg;
+        $apath =~ s/\s+//mgo;
         $ok = $self->setAlias($token, $apath) ? $ok:0;
     }
 
