@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: DB.pm,v 1.4 2005/11/07 17:18:27 ssinyagin Exp $
+# $Id: DB.pm,v 1.5 2006/10/08 21:05:21 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::DB;
@@ -278,6 +278,34 @@ sub c_del
 }
 
 
+sub c_get
+{
+    my $self = shift;
+    my $cursor = shift;
+    my $key = shift;
+    my $val = undef;
+
+    if( $cursor->c_get( $key, $val, DB_SET ) == 0 )
+    {
+        return $val;
+    }
+    else
+    {
+        return undef;
+    }
+}
+
+sub c_put
+{
+    my $self = shift;
+    my $cursor = shift;
+    my $key = shift;
+    my $val = shift;
+
+    return ( $cursor->c_put( $key, $val, DB_KEYFIRST ) == 0 );
+}
+
+
 
 # Btree best match. We assume that the searchKey is longer or equal
 # than the matched key in the database.
