@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: DevDiscover.pm,v 1.21 2006/07/20 09:34:28 ssinyagin Exp $
+# $Id: DevDiscover.pm,v 1.24 2006/12/14 08:02:26 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Core SNMP device discovery module
@@ -69,6 +69,8 @@ our @copyParams =
       'monitor-timeoffset',
       'snmp-host',
       'snmp-port',
+      'snmp-localaddr',
+      'snmp-localport',
       'snmp-community',
       'snmp-version',
       'snmp-username',
@@ -143,7 +145,7 @@ sub discover
 
     foreach my $param ( @requiredParams )
     {
-        if( not $devdetails->param( $param ) )
+        if( not defined( $devdetails->param( $param ) ) )
         {
             Error('Required parameter not defined: ' . $param);
             return 0;
@@ -155,9 +157,12 @@ sub discover
     my $version = $devdetails->param( 'snmp-version' );
     $snmpargs{'-version'} = $version;    
 
-    foreach my $arg ( qw(-port -timeout -retries) )
+    foreach my $arg ( qw(-port -localaddr -localport -timeout -retries) )
     {
-        $snmpargs{$arg} = $devdetails->param( 'snmp' . $arg );
+        if( defined( $devdetails->param( 'snmp' . $arg ) ) )
+        {
+            $snmpargs{$arg} = $devdetails->param( 'snmp' . $arg );
+        }
     }
     
     if( $version eq '1' or $version eq '2c' )
