@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RFC2863_IF_MIB.pm,v 1.28 2006/05/17 12:37:25 ssinyagin Exp $
+# $Id: RFC2863_IF_MIB.pm,v 1.31 2006/12/05 16:59:26 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Standard IF_MIB discovery, which should apply to most devices
@@ -136,19 +136,26 @@ sub discover
     
     # Pre-populate the interfaces table, so that other modules may
     # delete unneeded interfaces
+    my $includeAdmDown =
+        $devdetails->param('RFC2863_IF_MIB::list-admindown-interfaces')
+        eq 'yes';
+    my $includeNotpresent =
+        $devdetails->param('RFC2863_IF_MIB::list-notpresent-interfaces')
+        eq 'yes';
+    my $excludeOperDown =
+        $devdetails->param('RFC2863_IF_MIB::exclude-down-interfaces')
+        eq 'yes';
     foreach my $ifIndex
         ( $devdetails->getSnmpIndices( $dd->oiddef('ifDescr') ) )
     {
-        if( ( $devdetails->param('RFC2863_IF_MIB::list-admindown-interfaces')
-              eq 'yes'
-              or
-              $devdetails->snmpVar($dd->oiddef('ifAdminStatus') .
-                                   '.' . $ifIndex) == 1 ) and
-            ( $devdetails->param('RFC2863_IF_MIB::list-notpresent-interfaces')
-              eq 'yes'
-              or
-              $devdetails->snmpVar($dd->oiddef('ifOperStatus') .
-                                   '.' . $ifIndex) != 6 ) )
+        my $admStatus =
+            $devdetails->snmpVar($dd->oiddef('ifAdminStatus') .'.'. $ifIndex);
+        my $operStatus =
+            $devdetails->snmpVar($dd->oiddef('ifOperStatus') .'.'. $ifIndex);
+        
+        if( ( $admStatus == 1 or $includeAdmDown ) and
+            ( $operStatus != 6 or $includeNotpresent ) and
+            ( $operStatus != 2 or not $excludeOperDown ) )
         {
             my $interface = {};
             $data->{'interfaces'}{$ifIndex} = $interface;
@@ -194,7 +201,8 @@ sub discover
     ## The capability 'interfaceIndexingManaged' disables the hints
     ## and lets the vendor discovery module to operate the indexing
     
-    if( not $devdetails->hasCap('interfaceIndexingManaged') )
+    if( not $devdetails->hasCap('interfaceIndexingManaged') and
+        not $devdetails->hasCap('interfaceIndexingPersistent') )
     {
         my $hint =
             $devdetails->param('RFC2863_IF_MIB::ifindex-map-hint');
@@ -221,8 +229,7 @@ sub discover
             }
             elsif( $hint eq 'ifIndex' )
             {
-                $data->{'param'}{'ifindex-map'} = '$IFIDX_IFINDEX';
-                storeIfIndexParams( $devdetails );
+                $devdetails->setCap('interfaceIndexingPersistent');
             }
             else
             {
@@ -231,6 +238,12 @@ sub discover
             }
         }
 
+        if( $devdetails->hasCap('interfaceIndexingPersistent') )
+        {
+            $data->{'param'}{'ifindex-map'} = '$IFIDX_IFINDEX';
+            storeIfIndexParams( $devdetails );
+        }
+            
         $hint =
             $devdetails->param('RFC2863_IF_MIB::subtree-name-hint');
         if( defined( $hint ) )
@@ -452,7 +465,19 @@ sub buildConfig
             $onlyName{$name} = 1;
         }
     }
-    
+
+    # Bandwidth usage
+    my %bandwidthLimits;
+    if( $devdetails->param('RFC2863_IF_MIB::bandwidth-usage') eq 'yes' )
+    {
+        my $limits = $devdetails->param('RFC2863_IF_MIB::bandwidth-limits');
+        foreach my $intfLimit ( split( /\s*;\s*/, $limits ) )
+        {
+            my( $intf, $limitIn, $limitOut ) = split( /\s*:\s*/, $intfLimit );
+            $bandwidthLimits{$intf}{'In'} = $limitIn;
+            $bandwidthLimits{$intf}{'Out'} = $limitOut;
+        }
+    }
 
     # tokenset member interfaces of the form
     # Format: tset:intf,intf; tokenset:intf,intf;
@@ -525,10 +550,17 @@ sub buildConfig
     }
     my $subtreeParams = {};
     my $subtreeComment = $devdetails->param('RFC2863_IF_MIB::subtree-comment');
+
     if( length( $subtreeComment ) > 0 )
     {
         $subtreeParams->{'comment'} = $subtreeComment;
     }
+
+    if( $devdetails->param('RFC2863_IF_MIB::bandwidth-usage') eq 'yes' )
+    {
+        $subtreeParams->{'overview-shortcuts'} = 'traffic,errors,bandwidth';
+    }
+    
     my $countersNode =
         $cb->addSubtree( $devNode, $subtreeName, $subtreeParams,
                          ['RFC2863_IF_MIB::rfc2863-ifmib-subtree'] );
@@ -685,6 +717,26 @@ sub buildConfig
             }
         }
 
+        if( $devdetails->param('RFC2863_IF_MIB::bandwidth-usage') eq 'yes' )
+        {
+            if( defined( $bandwidthLimits{$subtreeName} ) )
+            {
+                $interface->{'param'}{'bandwidth-limit-in'} =
+                    $bandwidthLimits{$subtreeName}{'In'};
+                $interface->{'param'}{'bandwidth-limit-out'} =
+                    $bandwidthLimits{$subtreeName}{'Out'};
+            }
+
+            # We accept that parameters may be added by some other ways
+
+            if( defined( $interface->{'param'}{'bandwidth-limit-in'} ) and
+                defined( $interface->{'param'}{'bandwidth-limit-out'} ) )
+            {
+                push( @templates,
+                      'RFC2863_IF_MIB::interface-bandwidth-usage' );
+            }
+        }
+
         if( ref( $interface->{'templates'} ) )
         {
             push( @templates, @{$interface->{'templates'}} );
