@@ -16,7 +16,7 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RPN.pm,v 1.8 2006/01/06 10:41:09 ssinyagin Exp $
+# $Id: RPN.pm,v 1.9 2006/09/29 07:55:58 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # a simple little RPN calculator -- implements the same operations
@@ -52,7 +52,7 @@ my $operators = {
     'GT' => [ 2, sub{ ($_[0] >  $_[1]) ? 1:0; } ],
     'GE' => [ 2, sub{ ($_[0] >= $_[1]) ? 1:0; } ],
     'EQ' => [ 2, sub{ ($_[0] == $_[1]) ? 1:0; } ],
-    'IF' => [ 3, sub{ $_[0] ? $_[1] : $_[2]; } ],
+    'IF' => [ 3, sub{ defined($_[0]) ? ($_[0] ? $_[1] : $_[2]) : undef; }, 1],
     'MIN' => [ 2, sub{ ($_[0] <  $_[1]) ? $_[0] : $_[1]; } ],
     'MAX' => [ 2, sub{ ($_[0] >  $_[1]) ? $_[0] : $_[1]; } ],
     'UN'   => [ 1, sub{ defined($_[0]) ? $_[0]->is_nan() : 1; }, 1 ],
